package cn.iocoder.yudao.module.delivery.controller.admin;


import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.json.JsonUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.delivery.entity.*;
import cn.iocoder.yudao.module.delivery.entity.bar.CustomerReportBar;
import cn.iocoder.yudao.module.delivery.entity.bar.series;
import cn.iocoder.yudao.module.delivery.entity.bar.yAxis;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService;
import cn.iocoder.yudao.module.delivery.service.CustomerReportService;
import cn.iocoder.yudao.module.delivery.service.EcwReportPermissionService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.time.YearMonth;
import java.util.*;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * (CustomerReport)客户看板报表
 *
 * @author lanbm
 * @since 2024-03-27
 */
@RestController
@RequestMapping("/Report/CustomerReport")
public class CustomerReportControl {

    @Autowired
    private CustomerReportService customerReportService;

    //报表权限信息
    @Resource
    private EcwReportPermissionService ecwReportPermissionService;


    private String getCurDate() {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String currentDate = sdf.format(date);
        return currentDate;
    }

    private int getCurYear() {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        return year;
    }

    /*
     获取某年某月的最后一天 lanbm 2024-04-16 add
     */
    private int getLastDay(int year, int month) {
        YearMonth yearMonth = YearMonth.of(year, month);
        int lastDay = yearMonth.lengthOfMonth();
        return lastDay;
    }

    private CustomerReportReq getReq(CustomerReportReq Req) {
        int y = getCurYear();
        if (Req.getDuibiYear() == null) {
            Req.setDuibiYear(String.valueOf(y - 1));
        }

        if (Req.getSDate() == null) {
            Req.setCurYear(String.valueOf(y));
            Req.setSDate(String.valueOf(y) + "-01-01");
            Req.setSDuiBiDate(Req.getDuibiYear() + "-01-01");

        } else {
            //2024-03
            String sT = Req.getSDate();
            Req.setSDate(sT + "-01");

            String s = sT.substring(5, 7);
            Req.setSDuiBiDate(Req.getDuibiYear() + "-" + s + "-01");

            Req.setCurYear(sT.substring(0, 4));
        }

        if (Req.getEDate() == null) {
            Req.setEDate(getCurDate());

            String s = Req.getEDate().substring(5, 7);
            int lastDay = getLastDay(Integer.parseInt(Req.getDuibiYear()), Integer.parseInt(s));
            Req.setEDuiBiDate(Req.getDuibiYear() + "-" + s + "-" + String.valueOf(lastDay));

        } else {
            String sE = Req.getEDate();
            String sY = sE.substring(0, 4);
            String s = sE.substring(5, 7);
            int lastDay = getLastDay(Integer.parseInt(sY), Integer.parseInt(s));
            Req.setEDate(sE + "-" + String.valueOf(lastDay));

            lastDay = getLastDay(Integer.parseInt(Req.getDuibiYear()), Integer.parseInt(s));
            Req.setEDuiBiDate(Req.getDuibiYear() + "-" + s + "-" + String.valueOf(lastDay));
        }
        return Req;
    }

    /*
    计算百分比
    lanbm 2024-04-10 add
    */
    private String getPercentage(BigDecimal part, BigDecimal total) {
        BigDecimal percentage = part.divide(total, 2,
                BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
        // 输出百分比
        String sR = format(percentage) + "%";
        return sR;
    }

    /*
     格式化BigDecimal为字符串，保留两位小数
     lanbm 2024-04-10 add
     */
    private String format(BigDecimal value) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMinimumFractionDigits(2);
        numberFormat.setMaximumFractionDigits(2);
        return numberFormat.format(value.doubleValue());
    }


    /*
     客户分析列表 lanbm 2024-04-02 add
     xml动态查询参考网址
     https://blog.csdn.net/CYK_byte/article/details/128611104
     */
    @PostMapping("/getPageList")
    public CommonResult<CustomerReportBar> getListPage(@Valid @RequestBody Map map) {
        //department 部门查询条件是客户经理所在的部门，
        //客户信息中有个 department 字段，但这个值都是空的
        System.out.println(map);
        //map转model对象
        CustomerReportReq Req = JSON.parseObject(JSON.toJSONString(map), CustomerReportReq.class);
        Req = getReq(Req);
        System.out.println("查询参数为：" + Req);


        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId, loginUserId);
        List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
        if (objList == null || objList.size() == 0) {
            ErrorCode eCode = new ErrorCode(2024041506,
                    "当前用户未分配报表权限，请联系系统管理员处理。");
            return error(eCode);
        }

        //把model转 JSON字符串 lanbm
        //String jsonStr = JsonUtils.toJsonString(Req);
        //JsonUtils.SaveLog(jsonStr);

        CustomerReportBar customerReportBar = new CustomerReportBar();

        PageResult<CustomerReportResp> pageResult =
                customerReportService.getListPage(Req);

        customerReportBar.setTotal(pageResult.getTotal());
        List<CustomerReportResp> lis = pageResult.getList();

        yAxis y = new yAxis();
        y.setType("category");
        y.setInverse(true);

        List<String> l = new ArrayList<>();

        int nT=1;
        for (CustomerReportResp cr : lis) {
            //top 加客户编号，客户名称
            l.add("Top "+String.valueOf(nT)+"_" +cr.getNumber()+"_"+cr.getName());
            //l.add("Top "+String.valueOf(nT));
            nT++;

            cr.setDuibiYear(Req.getDuibiYear());
        }
        y.setData(l);
        customerReportBar.setObjyAxis(y);

        List<series> sl = new ArrayList<>();

        series s1 = new series();
        s1.setName(Req.getCurYear());
        s1.setType("bar");
        s1.setBarGap("60%");
        s1.setBarCategoryGap("60%");
        s1.setMyTip("12");
        List<String> s1L = new ArrayList<>();

        for (CustomerReportResp cr : lis) {
            if (cr.getSumvolume1() == null) cr.setSumvolume1(BigDecimal.valueOf(0));
            if (cr.getSumweight3() == null) cr.setSumweight3(BigDecimal.valueOf(0));

            //cr.setAllsumvolume(cr.getSumvolume1().add(cr.getSumweight3()));

            if (cr.getAllsumvolume() == null) {
                cr.setAllsumvolume(new BigDecimal(0));
                s1L.add("0");
            } else {
                s1L.add(cr.getAllsumvolume().toString());
            }

            if (cr.getAllsumvolumeTb() == null ||
                    cr.getAllsumvolumeTb().compareTo(BigDecimal.ZERO) == 0) {
                cr.setAllsumvolumeTbMsg("同期值为0");
            }
            else
            {
                //本期减同期 除 同期
                String sS=getPercentage((cr.getAllsumvolume().subtract(cr.getAllsumvolumeTb())),
                        cr.getAllsumvolumeTb());
                cr.setAllsumvolumeTbMsg(sS);

            }
        }
        s1.setData(s1L);
        sl.add(s1);


        //对比数据
        /*
        series s2 = new series();
        s2.setName(Req.getDuibiYear());
        s2.setType("bar");
        s2.setBarGap("60%");
        s2.setBarCategoryGap("60%");
        List<String> s2L = new ArrayList<>();
        for (CustomerReportResp cr : lis) {
            if (cr.getAllsumvolumeTb() == null) {
                s2L.add("0");
            } else {
                s2L.add(cr.getAllsumvolumeTb().toString());
            }
        }
        s2.setData(s2L);
        sl.add(s2);*/

        customerReportBar.setObSseries(sl);
        customerReportBar.setResultList(lis);

        return success(customerReportBar);

    }


    @PostMapping("/getBarData")
    public CommonResult<CustomerReportBar>
    getBarData(@RequestBody CustomerReportReq Req) {
        CustomerReportBar customerReportBar = new CustomerReportBar();
        yAxis y = new yAxis();
        y.setType("category");
        List<String> l = new ArrayList<>();
        l.add("湖北");
        l.add("湖南");
        l.add("广东");
        y.setData(l);
        customerReportBar.setObjyAxis(y);

        List<series> sl = new ArrayList<>();

        series s1 = new series();
        s1.setName("2024");
        s1.setType("bar");

        List<String> s1L = new ArrayList<>();
        s1L.add("12");
        s1L.add("15");
        s1L.add("20");
        s1.setData(s1L);

        sl.add(s1);


        series s2 = new series();
        s2.setName("2023");
        s2.setType("bar");

        List<String> s2L = new ArrayList<>();
        s2L.add("40");
        s2L.add("50");
        s2L.add("60");
        s2.setData(s2L);

        sl.add(s2);

        customerReportBar.setObSseries(sl);

        return success(customerReportBar);
    }


    @PostMapping("/exportExcel")
    @ApiOperation("导出客户看板数据")
    @OperateLog(type = EXPORT)
    public void export(HttpServletResponse response, @Valid CustomerReportReq Req) throws IOException {
        // List<DictDataDO> list = dictDataService.getDictDatas(reqVO);
        //List<DictDataExcelVO> data = DictDataConvert.INSTANCE.convertList02(list);
        // 输出
        //ExcelUtils.write(response, "客户分析数据.xls", "客户分析数据列表", DictDataExcelVO.class, data);
    }
}
