package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.delivery.entity.EcwVz;
import cn.iocoder.yudao.module.delivery.entity.EcwVzPageReq;
import cn.iocoder.yudao.module.delivery.service.EcwVzService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import javax.validation.*;
import io.swagger.v3.oas.annotations.Operation;

/**
 * V值转换配置参数(EcwVz)表控制层
 *
 * @author lanbm
 * @since 2024-04-02
 */
@RestController
@RequestMapping("/ecwVz/vz")
public class EcwVzController {
    /**
     * 服务对象
     */
    @Resource
    private EcwVzService ecwVzService;

    private final String HYPG = "海运拼柜";
    private final String ZXKY = "专线空运";

    /**
     * 新增数据
     *
     * @param ecwVz 实体
     * @return 新增结果
     */
    @PostMapping("/create")
    public CommonResult<Long> create(@Valid @RequestBody EcwVz ecwVz) {
        System.out.println(ecwVz);
        EcwVz temp = ecwVzService.getOne(new LambdaQueryWrapperX<EcwVz>()
                .eqIfPresent(EcwVz::getTitleZh, ecwVz.getTitleZh()));

        if (temp == null) {
            return success(ecwVzService.create(ecwVz));
        } else {
            ecwVz.setId(temp.getId());
            ecwVzService.update(ecwVz);
            return success(ecwVz.getId());
        }
    }

    /**
     * 编辑数据
     *
     * @param ecwVz 实体
     * @return 编辑结果
     */
    @PostMapping("/update")
    public CommonResult<Boolean> update(@RequestBody EcwVz ecwVz) {
        //System.out.println(ecwVz);
        ecwVzService.update(ecwVz);
        return success(true);
    }


    /**
     * 删除数据
     *
     * @param id 主键
     * @return 删除是否成功
     */

    @GetMapping("/delete")
    public CommonResult<Boolean> delete(@RequestParam Long id) {
        //自定义物理删除方法
        ecwVzService.delete(id);
        return success(true);
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @GetMapping("/get")
    public CommonResult<EcwVz> getModel(@RequestParam("id") Long id) {
        EcwVz ecwVz = ecwVzService.getModel(id);
        return success(ecwVz);
    }

    /**
     * 查询所有数据
     *
     * @return 实例对象集合
     */
    @GetMapping
    public CommonResult queryAll() {
        return success(this.ecwVzService.list());
    }

    /**
     * 分页查询
     *
     * @return 分页对象集合
     */

    @GetMapping("/page")
    @Operation(summary = "获取分页查询列表")
    public CommonResult<PageResult<EcwVz>> selectPage(@Valid EcwVzPageReq PageReq) {
        return success(this.ecwVzService.selectPage(PageReq));
    }

}

