package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.delivery.entity.*;
import cn.iocoder.yudao.module.delivery.entity.bar.CustomerReportBar;
import cn.iocoder.yudao.module.delivery.entity.bar.series;
import cn.iocoder.yudao.module.delivery.entity.bar.yAxis;
import cn.iocoder.yudao.module.delivery.service.SalesReportService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

/**
 * (Employee)表控制层
 *
 * @author lanbm
 * @since 2024-03-27 21:25:55
 */
@RestController
@Api(tags = "管理后台 -统计报表")
@RequestMapping("/Report/SalesReport")
public class SalesReportControl {

    //ecw_order_departure 订单始发地
    //ecw_order_consignor 订单发货人
    //ecw_order_consignee 订单收货人

    //订单表 ecw_order
    //drawee 付款人 付款人：1 发货人 2 收货人 3 自定义
    //status 订单状态 0草稿 2待入仓，3入参中

    @Autowired
    private SalesReportService salesReportService;

    /*
     首次成交的客户数统计
     lanbm 2024-04-01
     */
    @PostMapping("/FirstCustomerCount")
    public CommonResult<Integer> FirstCustomerCount(@RequestBody SalesReportReq Req) {
        int nR = salesReportService.FirstCustomerCount(Req);
        return success(nR);
    }

    /*
    按运输方式统计重量和方数
    lanbm 2024-04-01
    */
    @PostMapping("/SalesReportCount")
    public CommonResult<SalesReportResp> SalesReportCount(@RequestBody SalesReportReq Req) {
        String s = "";
        SalesReportResp re = salesReportService.SalesReportCount(Req);
        return success(re);
    }

    /*
     图表分析统计结果
     lanbm 2024-04-01 add
     */
    @PostMapping("/SalesReportList")
    public CommonResult<Integer> SalesReportList(@RequestBody SalesReportReq Req) {
        int nR = salesReportService.FirstCustomerCount(Req);
        return success(nR);
    }

    /*
    获取销售分析图表数据
     */
    @GetMapping("/getEChartData")
    public CommonResult<CustomerReportBar>
    getEChartData(@Valid SalesAnalysisReq query) {

        CustomerReportBar customerReportBar = new CustomerReportBar();

        yAxis y = new yAxis();
        y.setType("category");
        y.setInverse(true);

        List<String> l = new ArrayList<>();

        l.add("1");
        l.add("2");
        l.add("3");
        l.add("4");
        l.add("5");
        y.setData(l);
        customerReportBar.setObjyAxis(y);

        List<series> sl = new ArrayList<>();

        //总V值
        series s1 = new series();
        s1.setName("2024");
        s1.setType("bar");
        s1.setBarGap("60%");
        s1.setBarCategoryGap("60%");
        List<String> s1L = new ArrayList<>();
        s1L.add("12");
        s1L.add("4");
        s1L.add("7");
        s1L.add("10");
        s1L.add("20");
        s1.setData(s1L);
        sl.add(s1);


        //海运
        series s2 = new series();
        s2.setName("2023");
        s2.setType("bar");
        s2.setBarGap("60%");
        s2.setBarCategoryGap("60%");
        List<String> s2L = new ArrayList<>();
        s2L.add("10");
        s2L.add("13");
        s2L.add("6");
        s2L.add("8");
        s2L.add("9");
        s2.setData(s2L);
        sl.add(s2);


        //空运
        series s3 = new series();
        s3.setName("2022");
        s3.setType("bar");
        s3.setBarGap("60%");
        s3.setBarCategoryGap("60%");
        List<String> s3L = new ArrayList<>();
        s3L.add("10");
        s3L.add("13");
        s3L.add("19");
        s3L.add("8");
        s3L.add("19");
        s3.setData(s3L);
        sl.add(s3);

        customerReportBar.setObSseries(sl);
        // customerReportBar.setResultList(lis);

        return success(customerReportBar);
    }


}
