package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.delivery.entity.EcwVz;
import cn.iocoder.yudao.module.delivery.entity.EcwVzPageReq;
import cn.iocoder.yudao.module.delivery.entity.deptex.DeptEx;
import cn.iocoder.yudao.module.delivery.mapper.DeptExMapper;
import cn.iocoder.yudao.module.system.controller.admin.user.vo.user.UserSimpleResp2VO;
import cn.iocoder.yudao.module.system.convert.user.UserConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.dept.DeptService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;

import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@RestController
@Api(tags = "管理后台 -统计报表")
@RequestMapping("/Report/SysDeptEx")
public class SysDeptExtControl {

    //lanbm 2024-04-19 添加的系统基础扩展功能

    @Resource
    private DeptExMapper deptExMapper;

    @Resource
    private AdminUserService userService;

    @Resource
    private DeptService deptService;

    @GetMapping("/getDeptChild")
    @Operation(summary = "根据父机部门获取子部门")
    public CommonResult<List<DeptEx>> getDeptChild(@Valid DeptEx Req) {
        return success(deptExMapper.getDeptChild(Req));
    }

    @GetMapping("/listServiceUser")
    @ApiOperation(value = "获取客服岗人员列表", notes = "只包含客服用户，主要用于前端的下拉选项")
    public CommonResult<List<UserSimpleResp2VO>> listServiceUser(@Valid DeptEx Req) {

        List<AdminUserDO> list =
                userService.getUsersByPostIds(Arrays.asList(2L, 19L));
        List<UserSimpleResp2VO> resultList =
                UserConvert.INSTANCE.convertList05(list);

        if (CollectionUtil.isNotEmpty(resultList)) {
            List<Long> deptIds =
                    resultList.stream().map(t -> t.getDeptId()).
                            filter(Objects::nonNull).distinct().
                            collect(Collectors.toList());

            Map<Long, DeptDO> deptMap = deptService.getDeptMap(deptIds);

            if (CollectionUtil.isNotEmpty(deptMap)) {
                for (UserSimpleResp2VO userSimpleResp2VO : resultList) {
                    if (userSimpleResp2VO.getDeptId() != null) {
                        DeptDO deptDO = deptMap.get(userSimpleResp2VO.getDeptId());
                        if (deptDO != null) {
                            userSimpleResp2VO.setDeptName(deptDO.getName());
                        }
                    }
                }
            }
        }

        List<DeptEx> listDept = deptExMapper.getDeptChild(Req);
        List<Long> deptQuanX = new ArrayList<>();
        for (DeptEx d : listDept
        ) {
            deptQuanX.add(d.getId());
        }
        List<UserSimpleResp2VO> list2 = new ArrayList<>();
        for (UserSimpleResp2VO u : resultList
        ) {
            if (deptQuanX.contains(u.getDeptId())) {
                list2.add(u);
            }
        }
        // 排序后，返回给前端
        return success(list2);
    }
}
