package cn.iocoder.yudao.module.delivery.listener;

import cn.iocoder.yudao.framework.apollo.core.event.Order.CalculateOrderVValueEvent;
import cn.iocoder.yudao.framework.apollo.core.event.Order.CalculateOrderYeJiTypeEvent;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.delivery.entity.EcwVz;
import cn.iocoder.yudao.module.delivery.service.EcwVzService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_EXPRESS_NUMBER_TOO_LONG;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_NOT_EXISTS;
import static cn.iocoder.yudao.module.delivery.enums.ErrorCodeConstants.VCONFIG_NOT_EXISTS;

/*
 计算订单V值计算  海运已装柜，空运已入仓 计算V值
 lanbm 2024-05-13 add
 */
@Component("CalculateOrderVValueListener")
@AllArgsConstructor
@Slf4j
public class CalculateOrderVValueListener {
    @Autowired
    private OrderService orderService;

    @Autowired
    private EcwVzService ecwVzService;

    @EventListener(CalculateOrderVValueEvent.class)
    public void CalculateOrderVValuePushEvent(
            CalculateOrderVValueEvent event) {

        OrderDO orderDO = orderService.getById(event.getOrderId());
        if (orderDO == null) {
            throw exception(ORDER_NOT_EXISTS);
        }
        //订单状态为已入仓之后的状态，其他状态是否排除，后续和客户确认，先按此逻辑处理
        int nS = orderDO.getStatus();
        if (nS >= 5) {
            //查询配置参数信息
            EcwVz temp = null;
            BigDecimal CalVvalue = new BigDecimal(0);
            if (orderDO.getTransportId() == 1) {
                //海运计算V值
                temp = ecwVzService.getOne(new LambdaQueryWrapperX<EcwVz>()
                        .eqIfPresent(EcwVz::getTitleZh, "M3"));
                if (temp == null) {
                    throw exception(VCONFIG_NOT_EXISTS);
                }
                CalVvalue = new BigDecimal(temp.getVz()).divide(orderDO.getSumVolume());
            } else if (orderDO.getTransportId() == 3) {
                //空运计算V值
                temp = ecwVzService.getOne(new LambdaQueryWrapperX<EcwVz>()
                        .eqIfPresent(EcwVz::getTitleZh, "KG"));
                if (temp == null) {
                    throw exception(VCONFIG_NOT_EXISTS);
                }
                CalVvalue = new BigDecimal(temp.getVz()).divide(orderDO.getSumWeight());
            }

            //计算业绩产生时间
            if (orderDO.getRucangTime() == null) {
                event.setYejiCreateDate(new Date());
            } else {
                event.setYejiCreateDate(orderDO.getRucangTime());
            }
            orderService.UpdateOrderVValue(event.getOrderId(), CalVvalue,
                    event.getYejiCreateDate());
        }

    }
}
