package cn.iocoder.yudao.module.delivery.listener;

import cn.iocoder.yudao.framework.apollo.core.event.Customer.CalculateCustomerTypeEvent;
import cn.iocoder.yudao.framework.apollo.core.event.Order.CalculateOrderYeJiTypeEvent;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.delivery.enums.ErrorCodeConstants.CUSTOMER_NOT_EXISTS;
import static cn.iocoder.yudao.module.order.enums.ErrorCodeConstants.ORDER_NOT_EXISTS;

/*
 计算订单业绩类型监听
 lanbm 2024-05-13 add
 */
@Component("CalculateOrderYeJiTypeListener")
@AllArgsConstructor
@Slf4j
public class CalculateOrderYeJiTypeListener {

    @Autowired
    private OrderService orderService;

    @Autowired
    private CustomerService customerService;

    @EventListener(CalculateOrderYeJiTypeEvent.class)
    public void CalculateOrderYeJiTypePushEvent(
            CalculateOrderYeJiTypeEvent event) {

        OrderDO orderDO = orderService.getById(event.getOrderId());
        if (orderDO == null) {
            throw exception(ORDER_NOT_EXISTS);
        }
        //获取订单中计算号的业绩归属客户ID
        event.setCustomerId(orderDO.getCustomerId());
        CustomerDO customerDO = customerService.getById(event.getCustomerId());
        if (customerDO == null) {
            throw exception(CUSTOMER_NOT_EXISTS);
        }

        int isNew = 0;
        if (customerDO.getIsNew() == true)
            isNew = 1;//新客户业绩类型
        else
            isNew = 0;//老客户业绩类型

        orderService.UpdateOrderYeJiType(event.getOrderId(), isNew);


    }
}
