package cn.iocoder.yudao.module.delivery.listener;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ReportCustomerAnalysisExcelExportEvent;
import cn.iocoder.yudao.framework.apollo.core.event.export.ReportSalesAnalysisExcelExportEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.delivery.convert.CustomerAnalysis;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisExcelResp;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisReq;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisResp;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;


@Component("CustomerAnalysisExcelExportListener")
@AllArgsConstructor
@Slf4j
public class CustomerAnalysisExcelExportListener {

    private UeProperties ueProperties;
    private FileService fileService;

    @Autowired
    private CustomerAnalysisService customerAnalysisService;

    /**
     * 客户分析导出监听 lanbm 2024-05-01 add
     *
     * @param event 编号记录
     */
    @EventListener(ReportCustomerAnalysisExcelExportEvent.class)
    public void CustomerAnalysisExcelExportPushEvent(
            ReportCustomerAnalysisExcelExportEvent event) {

        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {

                System.out.println("开始数据导出");

                DateTimeFormatter formatter =
                        DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().
                        concat("/CustomerAnalysis/excel/");

                File fileDir = new File(dir);
                if (!fileDir.exists()) {
                    fileDir.mkdirs();
                }
                CustomerAnalysisReq query =
                        JSONObject.parseObject(event.getRequestParams(),
                                CustomerAnalysisReq.class);
                String fileName =
                        event.getUserId().toString().
                                concat(StrUtil.DASHED).concat(
                                        event.getUserType().toString()).
                                concat(StrUtil.DASHED).
                                concat(nowTime).
                                concat("客户分析报表.xlsx");

                List<CustomerAnalysisResp> list =
                        customerAnalysisService.GetDataListResult(query);
                //把查询列表转换位Excel对象
                List<CustomerAnalysisExcelResp> listExel =
                        CustomerAnalysis.INSTANCE.convertList(list);
                String path = ExcelUtils.write(dir, fileName,
                        "客户分析报表",
                        CustomerAnalysisExcelResp.class,
                        listExel);
                // 获取到临时文件
                File file = new File(path);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(file);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) file.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }

    }

}
